@@ -436,10 +436,11 @@ class GoSlideLocal:
     def __init__(self, timeout=DEFAULT_TIMEOUT, authexception=True):
         """Create the object with required parameters."""
         self._timeout = timeout
-        self._cnoncecount = 0
         self._authexception = authexception
+        self._cnoncecount = 0
         self._requestcount = 0
-        self._slides = {}
+        self._slide_passwd = {}
+        self._slide_api = {}
 
     def _md5_utf8(self, x):
         if isinstance(x, str):
@@ -537,6 +538,7 @@ class GoSlideLocal:
                         jsondata = None
 
                     return resp.status, jsondata
+
                 if resp.status == 401:
 
                     if "WWW-Authenticate" in resp.headers:
@@ -574,7 +576,7 @@ class GoSlideLocal:
         except asyncio.TimeoutError as err:
             raise ClientTimeoutError("Connection Timeout") from None
 
-    async def _request(self, hostname, password, reqtype, uri, data=None):
+    async def _request(self, hostname, password, apiversion, reqtype, uri, data=None):
         """Digest authentication using dorequest."""
 
         # Local API uses digest authentication:
@@ -587,50 +589,81 @@ class GoSlideLocal:
         # format URL with hostname/ip and uri value
         url = "http://{}{}".format(hostname, uri)
 
-        # First request, should return a 401 error
-        respstatus, resptext = await self._dorequest(reqtype, url)
+        # First request, should return a 401 error for v1
+        # First request is not required for v2
 
-        # Only a 401 response is correct
-        if respstatus == 401:
+        # Default is version 1, when we do WWW-Authentication
+        if apiversion == 1:
 
-            # The resptext contains the WWW-Authentication header
-            auth = self._make_digest_auth("user", password, reqtype, uri, resptext)
+            # do request to obtain a WWW-authentication header:
+            respstatus, resptext = await self._dorequest(reqtype, url)
 
-            respstatus, resptext = await self._dorequest(
-                reqtype, url, digestauth=auth, data=data
-            )
+            # Only a 401 response is correct
+            if respstatus == 401:
+
+                # The resptext contains the WWW-Authentication header
+                auth = self._make_digest_auth("user", password, reqtype, uri, resptext)
+
+                respstatus, resptext = await self._dorequest(
+                    reqtype, url, digestauth=auth, data=data
+                )
+
+                if respstatus == 200:
+                    return resptext
+
+                # Anything else is an error
+                _LOGGER.error(
+                    "Failed request with Local API Digest Authentication challenge. HTTPCode=%s",
+                    respstatus,
+                )
+            else:
+                # We expected a 401 Digest Auth here
+                _LOGGER.error(
+                    "Failed request with Local API v1. Received HTTPCode=%s, expected HTTPCode=401. Maybe try switching to api version 2?",
+                    respstatus,
+                )
+
+        elif apiversion == 2:
+
+            respstatus, resptext = await self._dorequest(reqtype, url, data=data)
 
             if respstatus == 200:
                 return resptext
 
             # Anything else is an error
             _LOGGER.error(
-                "Failed request with Local API Digest Authentication challenge. HTTPCode=%s",
+                "Failed request Local API v2. HTTPCode=%s",
                 respstatus,
             )
+
         else:
-            # We expected a 401 Digest Auth here
             _LOGGER.error(
-                "Failed request with Local API. Received HTTPCode=%s, expected HTTPCode=401",
-                respstatus,
+                "Only v1 and v2 is supported.",
             )
 
         return None
 
-    async def slide_add(self, hostname, password):
+    async def slide_add(self, hostname, password, api=2):
         """Add slide to internal table, then you can use the local API."""
-        self._slides[hostname] = password
+        self._slide_passwd[hostname] = password
+        self._slide_api[hostname] = api
 
     async def slide_del(self, hostname):
         """Delete slide from internal table."""
-        if hostname in self._slides:
-            self._slides.remove(hostname)
+        if hostname in self._slide_passwd:
+            self._slide_passwd.remove(hostname)
+        if hostname in self._slide_api:
+            self._slide_api.remove(hostname)
         else:
             _LOGGER.error("Tried to delete none-existing '%s' from list", hostname)
 
+    async def slide_list(self):
+        """List all registered slides."""
+        return list(self._slide_passwd.keys())
+
     async def _slide_exist(self, hostname):
         """Function to check if slide exist in internal table."""
-        if hostname in self._slides:
+        if hostname in self._slide_passwd:
             return True
         else:
             _LOGGER.error(
@@ -658,7 +691,11 @@ class GoSlideLocal:
             return None
 
         result = await self._request(
-            hostname, self._slides[hostname], "POST", "/rpc/Slide.GetInfo"
+            hostname,
+            self._slide_passwd[hostname],
+            self._slide_api[hostname],
+            "POST",
+            "/rpc/Slide.GetInfo",
         )
 
         return result
@@ -691,7 +728,12 @@ class GoSlideLocal:
             return False
 
         resp = await self._request(
-            hostname, self._slides[hostname], "POST", "/rpc/Slide.SetPos", {"pos": pos}
+            hostname,
+            self._slide_passwd[hostname],
+            self._slide_api[hostname],
+            "POST",
+            "/rpc/Slide.SetPos",
+            data={"pos": pos},
         )
         return bool(resp)
 
@@ -701,7 +743,12 @@ class GoSlideLocal:
             return False
 
         resp = await self._request(
-            hostname, self._slides[hostname], "POST", "/rpc/Slide.SetPos", {"pos": 0.0}
+            hostname,
+            self._slide_passwd[hostname],
+            self._slide_api[hostname],
+            "POST",
+            "/rpc/Slide.SetPos",
+            data={"pos": 0.0},
         )
         return bool(resp)
 
@@ -711,7 +758,12 @@ class GoSlideLocal:
             return False
 
         resp = await self._request(
-            hostname, self._slides[hostname], "POST", "/rpc/Slide.SetPos", {"pos": 1.0}
+            hostname,
+            self._slide_passwd[hostname],
+            self._slide_api[hostname],
+            "POST",
+            "/rpc/Slide.SetPos",
+            data={"pos": 1.0},
         )
         return bool(resp)
 
@@ -721,7 +773,11 @@ class GoSlideLocal:
             return False
 
         resp = await self._request(
-            hostname, self._slides[hostname], "POST", "/rpc/Slide.Stop"
+            hostname,
+            self._slide_passwd[hostname],
+            self._slide_api[hostname],
+            "POST",
+            "/rpc/Slide.Stop",
         )
         return bool(resp)
 
@@ -731,7 +787,11 @@ class GoSlideLocal:
             return False
 
         resp = await self._request(
-            hostname, self._slides[hostname], "POST", "/rpc/Slide.Calibrate"
+            hostname,
+            self._slide_passwd[hostname],
+            self._slide_api[hostname],
+            "POST",
+            "/rpc/Slide.Calibrate",
         )
         return bool(resp)
 
@@ -742,9 +802,37 @@ class GoSlideLocal:
 
         resp = await self._request(
             hostname,
-            self._slides[hostname],
+            self._slide_passwd[hostname],
+            self._slide_api[hostname],
             "POST",
             "/rpc/Slide.Config.Wifi",
-            {"ssid": ssid, "pass": password},
+            data={"ssid": ssid, "pass": password},
+        )
+        return bool(resp)
+
+    async def slide_get_touchgo(self, hostname):
+        """Retrieve the slide TouchGo setting."""
+        result = await self.slide_info(hostname)
+        if result:
+            if "touch_go" in result:
+                return result["touch_go"]
+            _LOGGER.error(
+                "SlideGetTouchGo: Missing key 'touch_go' in JSON=%s", json.dumps(result)
+            )
+
+        return None
+
+    async def slide_set_touchgo(self, hostname, value):
+        """Change Touch-Go of a slide."""
+        if not await self._slide_exist(hostname):
+            return False
+
+        resp = await self._request(
+            hostname,
+            self._slide_passwd[hostname],
+            self._slide_api[hostname],
+            "POST",
+            "/rpc/Slide.touchGo",
+            data={"touch_go": value},
         )
         return bool(resp)
